//
//  Visopsys
//  Copyright (C) 1998-2014 J. Andrew McLaughlin
//
//  This program is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
//  wallpaper.c
//

// Calls the kernel's window manager to change the background image

/* This is the text that appears when a user requests help about this program
<help>

 -- wallpaper --

Set the background wallpaper image.

Usage:
  wallpaper [image_file]

(Only available in graphics mode)

This command will set the background wallpaper image from the (optional)
image file name parameter or, if no image file name is supplied, the program
will prompt the user.

Currently, bitmap (.bmp) and JPEG (.jpg) image formats are supported.

</help>
*/

#include <libintl.h>
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/api.h>
#include <sys/paths.h>
#include <sys/vsh.h>
#include <sys/window.h>

#define _(string) gettext(string)


int main(int argc, char *argv[])
{
	int status = 0;
	char fileName[MAX_PATH_NAME_LENGTH];

	setlocale(LC_ALL, getenv("LANG"));
	textdomain("wallpaper");

	// Only work in graphics mode
	if (!graphicsAreEnabled())
	{
		printf(_("\nThe \"%s\" command only works in graphics mode\n"), argv[0]);
		return (status = ERR_NOTINITIALIZED);
	}

	bzero(fileName, MAX_PATH_NAME_LENGTH);

	if (argc < 2)
	{
		// The user did not specify a file.  We will prompt them.
		status = windowNewFileDialog(NULL, _("Enter filename"),
			_("Please choose the background image:"), PATH_SYSTEM_WALLPAPER,
			fileName, MAX_PATH_NAME_LENGTH, 1);
		if (status != 1)
		{
			if (status == 0)
				return (status);

			printf("%s", _("No filename specified\n"));
			return (status);
		}
	}

	else
		strncpy(fileName, argv[1], MAX_PATH_NAME_LENGTH);

	if (strncmp(fileName, "none", MAX_PATH_NAME_LENGTH))
	{
		status = fileFind(fileName, NULL);
		if (status < 0)
		{
			printf("%s", _("File not found\n"));
			return (status);
		}

		status = windowShellTileBackground(fileName);
	}

	else
		status = windowShellTileBackground(NULL);

	return (status);
}

